/**************************************************************************/
/*                                                                        */
/*  This file is part of FISSC.                                           */
/*                                                                        */
/*  you can redistribute it and/or modify it under the terms of the GNU   */
/*  Lesser General Public License as published by the Free Software       */
/*  Foundation, version 3.0.                                              */
/*                                                                        */
/*  It is distributed in the hope that it will be useful,                 */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of        */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         */
/*  GNU Lesser General Public License for more details.                   */
/*                                                                        */
/*  See the GNU Lesser General Public License version 3.0                 */
/*  for more details (enclosed in the file LICENSE).                      */
/*                                                                        */
/**************************************************************************/

#include "interface.h"
#include "types.h"
#include "commons.h"
#include "random.h"

extern BOOL g_challengeFlag;
extern UBYTE g_challenge[CHALLENGE_SIZE];
extern UBYTE g_countermeasure;
extern UBYTE g_functionStack[MAX_STACK_INDEX];
extern UBYTE g_stackIndex;
extern UBYTE g_lastFunction;

#ifdef INLINE
__attribute__((always_inline)) inline void enterFunction(UBYTE id)
#else
void enterFunction(UBYTE id)
#endif
{
    UBYTE stepCounter = 0;
    if (g_functionStack[g_stackIndex] != NO_FUNCTION_ID) {
        countermeasure(1001);
    }
    stepCounter++; // 1
    g_functionStack[g_stackIndex] = id;
    stepCounter++; // 2
    g_stackIndex++;
    stepCounter++; // 3
    if (g_stackIndex >= MAX_STACK_INDEX) {
        countermeasure(1002);
    }
    stepCounter++; // 4
    if (g_functionStack[g_stackIndex-1] != id) {
        countermeasure(1003);
    }
    stepCounter++; // 5
    // step counter
    if (stepCounter != 5) {
        countermeasure(1004);
    }
}

#ifdef INLINE
__attribute__((always_inline)) inline void leaveFunction(UBYTE id)
#else
void leaveFunction(UBYTE id)
#endif
{
    UBYTE stepCounter = 0;
    if (g_functionStack[g_stackIndex-1] != id) {
        countermeasure(1005);
    }
    stepCounter++; // 1
    g_functionStack[g_stackIndex-1] = NO_FUNCTION_ID;
    stepCounter++; // 2
    // redundant check
    if (g_functionStack[g_stackIndex-1] != NO_FUNCTION_ID) {
        countermeasure(1006);
    }
    stepCounter++; // 3
    g_stackIndex--;
    stepCounter++; // 4
    // step counter
    if (stepCounter != 4) {
        countermeasure(1007);
    }
    g_lastFunction = id;
}

#ifdef INLINE
__attribute__((always_inline)) inline void checkLastFunctionA(UBYTE id)
#else
void checkLastFunctionA(UBYTE id)
#endif
{
    if (g_lastFunction != id) {
        countermeasure(1008);
    }
}


#ifdef INLINE
__attribute__((always_inline)) inline void checkLastFunctionB(UBYTE id)
#else
void checkLastFunctionB(UBYTE id)
#endif
{
    if (g_lastFunction != id) {
        countermeasure(1009);
    }
}

#ifdef INLINE
__attribute__((always_inline)) inline void generateRandom(UBYTE* buffer, UBYTE size, UBYTE* bufferCpy, UBYTE sizeCpy)
#else
void generateRandom(UBYTE* buffer, UBYTE size, UBYTE* bufferCpy, UBYTE sizeCpy)
#endif
{
    volatile UBYTE i;
    volatile UBYTE j;

    // Double Argument Step 2
    if ((buffer != bufferCpy) || (size != sizeCpy)) {
        countermeasure(1010);
    }

    // Virtual Stack step 1
    enterFunction(ID_GENERATE_RANDOM);

    // step counter
    j = size;
    for ( i = 0; i < size; i++ ) {
        buffer[i] = randomUBYTE();
        j--;
    };

    // loop counter and step counter
    if ((i != size) || (j != 0)) {
        countermeasure(1011);
    }

    // Virtual Stack step 2
    leaveFunction(ID_GENERATE_RANDOM);
}

BOOL getChallenge()
{
    g_challengeFlag = BOOL_FALSE;
    // Double Arguments
    generateRandom(g_challenge, CHALLENGE_SIZE, g_challenge, CHALLENGE_SIZE);
    // Virtual Stack step 3 (see step 1-2 in generateRandom)
    checkLastFunctionA(ID_GENERATE_RANDOM);
    g_challengeFlag = BOOL_TRUE;
    // Virtual Stack step 3 repeated
    checkLastFunctionB(ID_GENERATE_RANDOM);
    return g_challengeFlag;
}
