# Paper experimentations

This folder contains the source examples of the paper "Countermeasure Optimization in Multiple Fault-Injections Context", FDTC 20.

The experimentations uses five examples: 
 - *verifyPIN function*.
 - *Firmware updater*.
 - *GetChallenge function*.
 - AES cipher.
 - AES AddRoundKey.

Each example is divided in multiple variant, described below.

The examples are divided in five sources folders. Each folders contains a `src` subfolder containing the sources and a `results` folders containing the analysis scripts (from Lazart).

Sources use **Lazart's** instrumentation rules, please remove any macro related to **Lazart** to test normal execution.

For any question, please contact *etienne.boespflug@univ-grenoble-alpes.fr*.

## verifyPIN

It is the verifyPIN examples, with *Test duplication* and *ST's countermeasures* analysis.

**oracle()**: `g_authenticated == 1`.
**input**: userPIN == "symbolic" and cardPIN == "symbolic".

The following variants are used:

- *VP - TD*: test duplication:
 - analysis file: `ti-td-ccpa.py`.
- *VP - SSCF*: SecSwift CF countermeasure:
 - analysis file: `ti-swift-ccpa.py`.
- *VP - LBH*: LBH countermeasure:
 - analysis file: `ti-lbh-ccpa.py`.

## FirmwareUpdater

Contains the *Firmware Updater* example.

**oracle()**: `triggered==1 && (iscorrupted==1 || loadAddress != LOAD_ADDRESS)`
**input**: ̀`requiresUpdate`= symbolic, `pagecorrect`= symbolic.

The following variants are used:

- *FU - TD*: test duplication:
 - analysis file: `ti-td-ccpa.py`.
- *FU - SSCF*: SecSwift CF countermeasure:
 - analysis file: `ti-swift-ccpa.py`.

## GetChallenge

Contains the *GetChallenge* example.

**oracle()**: `g_countermeasure != 0) && ((g_challengeFlag != BOOL_TRUE) || (diff == 0))`
**input**: ̀`challenge`= FALSE, `initialValue`= `0xFF`, random=symbolic.

The following variants are used:

- *GC*: with integrated CCPs:
 - analysis file: `ti-ccpa.py`.
- *GC - TD*: test duplication:
 - analysis file: `ti-td-ccpa.py`.
- *GC - SSCF*: SecSwift CF countermeasure:
 - analysis file: `ti-swift-ccpa.py`.


## AES/AddRoundKey

Contains the *AES AddRoundKey step* example.

**oracle()**: `diff == 0`.
**input**: ̀`buf`= {0}, `key`= "1234...".

The following variants are used:

- *ARK - TD*: test duplication:
 - analysis file: `ti-td-ccpa.py`.
- *ARK - SSCF*: SecSwift CF countermeasure:
 - analysis file: `ti-swift-ccpa.py`.


## AES/AES Cipher

Contains the *AES cipher* example.

**oracle()**: `diff == 0`.

The following variants are used:

- *AES - TD*: test duplication:
 - analysis file: `ti-td-ccpa.py`.
- *AES - SSCF*: SecSwift CF countermeasure:
 - analysis file: `ti-swift-ccpa.py`.